@@ -26,7 +26,3 @@ else
     exit 1
 fi
 
-# Respawn the apps so they pick up the new db. Some apps ignore SIGTERM,
-# so we'll send them a signal they can't refuse.
-PIDS=`pidof $APPS` && kill -9 $PIDS
-
