@@ -42,11 +42,15 @@
 /* Configuration directory where EventLogger database and config is stored */
 #define CONFIG_DIR      ".rtcom-eventlogger"
 /* SQLite database where log is stored */
-#define SQLITE_DATABASE        "el.db"
+#define SQLITE_DATABASE        "el-v1.db"
+#define OLD_SQLITE_DATABASE    "el.db"
 #define ATTACH_DIR             "attachments"
 
 #define DBUS_PATH              "/rtcomeventlogger/signal"
 #define DBUS_INTERFACE         "rtcomeventlogger.signal"
+#define DBUS_MATCH             "type='signal'," \
+            "interface='rtcomeventlogger.signal'"
+
 
 /* We'll be using 1s granularity, so value of 2 here actually means
  * "somewhere between 1 and 3". */
@@ -192,7 +196,7 @@ static void rtcom_el_get_property(
  */
 static gboolean _ensure_db(RTComEl *el, gboolean reopen)
 {
-    gchar *fn;
+    gchar *fn, *old_fn;
 
     RTComElPrivate * priv = RTCOM_EL_GET_PRIV(el);
 
@@ -211,6 +215,7 @@ static gboolean _ensure_db(RTComEl *el, gboolean reopen)
             g_warning ("%s: can't create directory '%s': %s", G_STRFUNC,
                 fn, g_strerror (errno));
             g_free (fn);
+            return FALSE;
         }
     }
     g_free (fn);
@@ -218,6 +223,16 @@ static gboolean _ensure_db(RTComEl *el, gboolean reopen)
     fn = g_build_filename (el_get_home_dir (), CONFIG_DIR, SQLITE_DATABASE,
         NULL);
 
+    old_fn = g_build_filename (el_get_home_dir (), CONFIG_DIR,
+        OLD_SQLITE_DATABASE, NULL);
+
+    /* If needed, this will convert the v0 database format into the
+     * one we use today. */
+    rtcom_el_db_convert_from_db0 (fn, old_fn);
+
+    g_free (old_fn);
+
+    /* Now we can open the database. */
     priv->db = rtcom_el_db_open (fn);
     g_free (fn);
 
@@ -281,11 +296,7 @@ static void rtcom_el_init(
         return;
     }
 
-    dbus_bus_add_match(
-            priv->dbus,
-            "type='signal',"
-            "interface='rtcomeventlogger.signal'",
-            NULL);
+    dbus_bus_add_match(priv->dbus, DBUS_MATCH, NULL);
 
     _ensure_db(el, TRUE);
 }
@@ -299,6 +310,8 @@ rtcom_el_dispose (GObject *object)
 
     if (priv->dbus != NULL)
     {
+        dbus_bus_remove_match(priv->dbus, DBUS_MATCH, NULL);
+
         dbus_connection_remove_filter(
             priv->dbus,
             (DBusHandleMessageFunction) _dbus_filter_callback,
@@ -434,6 +447,23 @@ RTComEl * rtcom_el_new()
     return RTCOM_EL(g_object_new(RTCOM_TYPE_EL, NULL));
 }
 
+RTComEl * rtcom_el_get_shared ()
+{
+    static gpointer el = NULL;
+
+    if (G_UNLIKELY (NULL == el))
+    {
+        el = rtcom_el_new ();
+        g_object_add_weak_pointer (G_OBJECT (el), &el);
+    }
+    else
+    {
+        g_object_ref (G_OBJECT (el));
+    }
+
+    return el;
+}
+
 struct remotes_ctx {
     gboolean exists;
     gchar *abook_uid;
@@ -451,15 +481,11 @@ void _fetch_remote_data (gpointer data, gpointer user_data)
 }
 
 gboolean
-rtcom_el_update_remote_contact(RTComEl *el,
-    const gchar *local_uid, const gchar *remote_uid,
-    const gchar *new_abook_uid, const gchar *new_remote_name,
-    GError **error)
+rtcom_el_update_remote_contacts(RTComEl *el, GList *contacts, GError **error)
 {
     RTComElPrivate *priv = RTCOM_EL_GET_PRIV (el);
 
-    g_debug ("%s: called for (%s,%s) to set euid to: %s", G_STRFUNC,
-        local_uid, remote_uid, new_abook_uid);
+    g_return_val_if_fail (contacts != NULL, TRUE);
 
     if (!_ensure_db (el, TRUE))
     {
@@ -473,14 +499,91 @@ rtcom_el_update_remote_contact(RTComEl *el,
         return FALSE;
     }
 
-    if (!rtcom_el_db_exec_printf (priv->db, NULL, NULL, error,
-        "UPDATE Remotes SET abook_uid = %Q, remote_name = %Q "
-          "WHERE remote_uid = %Q AND local_uid = %Q;",
-        new_abook_uid, new_remote_name,
-        remote_uid, local_uid))
+    while (contacts != NULL)
     {
+        RTComElRemote *c = contacts->data;
+
+        if (!rtcom_el_db_exec_printf (priv->db, NULL, NULL, error,
+            "UPDATE Remotes SET abook_uid = %Q, remote_name = %Q "
+              "WHERE remote_uid = %Q AND local_uid = %Q;",
+            c->abook_uid, c->remote_name,
+            c->remote_uid, c->local_uid))
+        {
+            rtcom_el_db_rollback (priv->db, NULL);
+            return FALSE;
+        }
+
+        contacts = contacts->next;
+    }
+
+    if (!rtcom_el_db_commit (priv->db, error))
+    {
         rtcom_el_db_rollback (priv->db, NULL);
         return FALSE;
+    }
+
+    _emit_dbus(el, "RefreshHint", -1, NULL);
+
+    return TRUE;
+}
+
+
+gboolean
+rtcom_el_update_remote_contact(RTComEl *el,
+    const gchar *local_uid, const gchar *remote_uid,
+    const gchar *new_abook_uid, const gchar *new_remote_name,
+    GError **error)
+{
+    GList *li;
+    RTComElRemote c;
+    gboolean ret;
+
+    c.local_uid = (gchar *) local_uid;
+    c.remote_uid = (gchar *) remote_uid;
+    c.abook_uid = (gchar *) new_abook_uid;
+    c.remote_name = (gchar *) new_remote_name;
+
+    li = g_list_prepend (NULL, &c);
+
+    ret = rtcom_el_update_remote_contacts (el, li, error);
+
+    g_list_free (li);
+
+    return ret;
+}
+
+gboolean
+rtcom_el_remove_abook_uids (RTComEl *el, GList *abook_uids, GError **error)
+{
+    RTComElPrivate *priv = RTCOM_EL_GET_PRIV (el);
+
+    g_return_val_if_fail (abook_uids != NULL, TRUE);
+
+    if (!_ensure_db (el, TRUE))
+    {
+        g_set_error(error, RTCOM_EL_ERROR, RTCOM_EL_INTERNAL_ERROR,
+            "Can't update contact data, database isn't opened.");
+        return FALSE;
+    }
+
+    if (!rtcom_el_db_transaction (priv->db, FALSE, error))
+    {
+        return FALSE;
+    }
+
+    while (abook_uids != NULL)
+    {
+        const gchar *uid = abook_uids->data;
+
+        if (!rtcom_el_db_exec_printf (priv->db, NULL, NULL, error,
+            "UPDATE Remotes SET abook_uid = NULL WHERE abook_uid = %Q;",
+            uid))
+        {
+            rtcom_el_db_rollback (priv->db, NULL);
+            return FALSE;
+        }
+
+        abook_uids = abook_uids->next;
     }
 
     if (!rtcom_el_db_commit (priv->db, error))
@@ -489,8 +592,6 @@ rtcom_el_update_remote_contact(RTComEl *el,
         return FALSE;
     }
 
-    /* There are possibly a lot of events changed by this data change,
-     * so we have to tell the users to refresh their database models. */
     _emit_dbus(el, "RefreshHint", -1, NULL);
 
     return TRUE;
@@ -500,41 +601,16 @@ gboolean
 rtcom_el_remove_abook_uid (RTComEl *el, const gchar *abook_uid,
     GError **error)
 {
-    RTComElPrivate *priv = RTCOM_EL_GET_PRIV (el);
+    GList *li;
+    gboolean ret;
 
-    g_debug ("%s: clearing %s", G_STRFUNC, abook_uid);
+    li = g_list_prepend (NULL, (gchar *) abook_uid);
 
-    if (!_ensure_db (el, TRUE))
-    {
-        g_set_error(error, RTCOM_EL_ERROR, RTCOM_EL_INTERNAL_ERROR,
-            "Can't update contact data, database isn't opened.");
-        return FALSE;
-    }
+    ret = rtcom_el_remove_abook_uids (el, li, error);
 
-    if (!rtcom_el_db_transaction (priv->db, FALSE, error))
-    {
-        return FALSE;
-    }
+    g_list_free (li);
 
-    if (!rtcom_el_db_exec_printf (priv->db, NULL, NULL, error,
-        "UPDATE Remotes SET abook_uid = NULL WHERE abook_uid = %Q;",
-        abook_uid))
-    {
-        rtcom_el_db_rollback (priv->db, NULL);
-        return FALSE;
-    }
-
-    if (!rtcom_el_db_commit (priv->db, error))
-    {
-        rtcom_el_db_rollback (priv->db, NULL);
-        return FALSE;
-    }
-
-    /* There are possibly a lot of events changed by this data change,
-     * so we have to tell the users to refresh their database models. */
-    _emit_dbus(el, "RefreshHint", -1, NULL);
-
-    return TRUE;
+    return ret;
 }
 
 
@@ -629,9 +705,43 @@ _add_event_precheck(
         return FALSE;
     }
 
-    *eventtype_id = rtcom_el_get_eventtype_id(
-            el,
-            RTCOM_EL_EVENT_GET_FIELD(ev, event_type));
+    /* FIXME: these are temporary backwards-compatibility fixes
+     * so that old messaging-ui can be used with us. To be removed
+     * in a few releases. */
+    {
+      gchar *eventtype = RTCOM_EL_EVENT_GET_FIELD(ev, event_type);
+
+      if (!g_strcmp0 (eventtype, "RTCOM_EL_EVENTTYPE_CHAT_INBOUND"))
+        {
+          g_warning ("%s: Event type %s is deprecated!", G_STRFUNC, eventtype);
+          RTCOM_EL_EVENT_SET_FIELD(ev, event_type,
+              "RTCOM_EL_EVENTTYPE_CHAT_MESSAGE");
+        }
+      else if (!g_strcmp0 (eventtype, "RTCOM_EL_EVENTTYPE_CHAT_OUTBOUND"))
+        {
+          g_warning ("%s: Event type %s is deprecated!", G_STRFUNC, eventtype);
+          RTCOM_EL_EVENT_SET_FIELD(ev, event_type,
+              "RTCOM_EL_EVENTTYPE_CHAT_MESSAGE");
+          RTCOM_EL_EVENT_SET_FIELD(ev, outgoing, TRUE);
+        }
+      else if (!g_strcmp0 (eventtype, "RTCOM_EL_EVENTTYPE_SMS_INBOUND"))
+        {
+          g_warning ("%s: Event type %s is deprecated!", G_STRFUNC, eventtype);
+          RTCOM_EL_EVENT_SET_FIELD(ev, event_type,
+              "RTCOM_EL_EVENTTYPE_SMS_MESSAGE");
+        }
+      else if (!g_strcmp0 (eventtype, "RTCOM_EL_EVENTTYPE_SMS_OUTBOUND"))
+        {
+          g_warning ("%s: Event type %s is deprecated!", G_STRFUNC, eventtype);
+          RTCOM_EL_EVENT_SET_FIELD(ev, event_type,
+              "RTCOM_EL_EVENTTYPE_SMS_MESSAGE");
+          RTCOM_EL_EVENT_SET_FIELD(ev, outgoing, TRUE);
+        }
+    }
+
+    *eventtype_id = rtcom_el_get_eventtype_id(el,
+        RTCOM_EL_EVENT_GET_FIELD(ev, event_type));
+
     if(*eventtype_id == -1)
     {
         g_warning("EventType not found.");
@@ -702,11 +812,11 @@ _add_event_core(
     if (!rtcom_el_db_exec_printf (priv->db, NULL, NULL, NULL,
         "INSERT INTO Events ("
         "service_id, event_type_id, "
-        "storage_time, start_time, end_time, is_read, "
+        "storage_time, start_time, end_time, is_read, outgoing, "
         "flags, bytes_sent, bytes_received, "
         "local_uid, local_name, remote_uid, "
         "channel, free_text, group_uid) VALUES ( "
-        "%d, %d, %d, %d, %d, %d, %d, %d, %d, "
+        "%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, "
         "%Q, %Q, %Q, %Q, %Q, %Q);",
         service_id,
         eventtype_id,
@@ -714,6 +824,7 @@ _add_event_core(
         RTCOM_EL_EVENT_IS_SET(ev, start_time) ? RTCOM_EL_EVENT_GET_FIELD(ev, start_time) : 0,
         RTCOM_EL_EVENT_IS_SET(ev, end_time) ? RTCOM_EL_EVENT_GET_FIELD(ev, end_time): 0,
         RTCOM_EL_EVENT_IS_SET(ev, is_read) ? RTCOM_EL_EVENT_GET_FIELD(ev, is_read): 0,
+        RTCOM_EL_EVENT_IS_SET(ev, outgoing) ? RTCOM_EL_EVENT_GET_FIELD(ev, outgoing): 0,
         RTCOM_EL_EVENT_IS_SET(ev, flags) ? RTCOM_EL_EVENT_GET_FIELD(ev, flags) : 0,
         RTCOM_EL_EVENT_IS_SET(ev, bytes_sent) ? RTCOM_EL_EVENT_GET_FIELD(ev, bytes_sent) : 0,
         RTCOM_EL_EVENT_IS_SET(ev, bytes_received) ? RTCOM_EL_EVENT_GET_FIELD(ev, bytes_received) : 0,
@@ -1526,7 +1637,6 @@ RTComElIter * _get_events_core(
             "query", query,
             "sqlite3-database", priv->db,
             "sqlite3-statement", stmt,
-            "sql", sql,
             "plugins-table", priv->plugins,
             "atomic", atomic,
             NULL);
@@ -2584,6 +2694,7 @@ _init_plugin (
     GList * event_types = NULL;
     RTComElEventType * event_type = NULL;
     GList * flags = NULL;
+    GList *li;
     RTComElFlag * flag = NULL;
     sqlite3 * db = NULL;
     gpointer p;
@@ -2683,9 +2794,10 @@ _init_plugin (
     if(plugin->get_event_types)
     {
         event_types = plugin->get_event_types();
-        if(event_types) do
+
+        for (li = event_types; li; li = li->next)
         {
-            event_type = (RTComElEventType *) event_types->data;
+            event_type = (RTComElEventType *) li->data;
             p = g_hash_table_lookup(priv->event_types, event_type->name);
             if(p)
             {
@@ -2714,7 +2826,7 @@ _init_plugin (
             event_type->id = sqlite3_last_insert_rowid(db);
             g_hash_table_insert(priv->event_types, g_strdup(event_type->name), GINT_TO_POINTER(event_type->id));
             g_debug("EventType %s inserted.", event_type->name);
-        } while((event_types = g_list_next(event_types)));
+        }
 
         g_list_foreach(event_types, (GFunc) rtcom_el_eventtype_free, NULL);
         g_list_free(event_types);
@@ -2726,9 +2838,11 @@ _init_plugin (
     if(plugin->get_flags)
     {
         flags = plugin->get_flags();
-        if(flags) do
+
+        for (li = flags; li; li = li->next)
         {
-            flag = (RTComElFlag *) flags->data;
+
+            flag = (RTComElFlag *) li->data;
             p = g_hash_table_lookup(priv->flags, flag->name);
             if(p)
             {
@@ -2757,7 +2871,7 @@ _init_plugin (
             flag->id = sqlite3_last_insert_rowid(db);
             g_hash_table_insert(priv->flags, g_strdup(flag->name), GINT_TO_POINTER(flag->id));
             g_debug("Flag %s inserted.", flag->name);
-        } while((flags = g_list_next(flags)));
+        }
 
         g_list_foreach(flags, (GFunc) rtcom_el_flag_free, NULL);
         g_list_free(flags);
