@@ -70,35 +70,8 @@
 #define REMOTE_UID_3 "ext-salvatore.iovene-3@nokia.com"
 #define REMOTE_NAME_3 "Salvatore Iovene 3"
 
-#define DROP_CORE_TABLES \
-        "DROP TABLE Attachments;\n" \
-        "DROP TABLE Events;\n" \
-        "DROP TABLE EventTypes;\n" \
-        "DROP TABLE Flags;\n" \
-        "DROP TABLE GroupCache;\n" \
-        "DROP TABLE Headers;\n" \
-        "DROP TABLE Plugins;\n" \
-        "DROP TABLE Remotes;\n" \
-        "DROP TABLE Services;\n" \
-
 static RTComEl *el = NULL;
 
-static void
-el_exec (const gchar *sql)
-{
-    sqlite3 *db;
-
-    g_assert (el != NULL);
-
-    g_object_get (el,
-            "db", &db,
-            NULL);
-
-    fail_unless (sqlite3_exec (db, sql, NULL, NULL, NULL)
-            == SQLITE_OK);
-}
-
-
 static RTComElEvent *
 event_new_lite (void)
 {
@@ -150,12 +123,16 @@ static void
 core_setup (void)
 {
     g_type_init ();
+    const gchar *home;
+    gchar *fn;
 
-    /* Clear the database */
-    el = rtcom_el_new ();
-    g_assert (el != NULL);
-    el_exec (DROP_CORE_TABLES);
-    g_object_unref (el);
+    home = g_getenv ("RTCOM_EL_HOME");
+    if (!home)
+        home = g_get_home_dir();
+
+    fn = g_build_filename (home, ".rtcom-eventlogger", "el-v1.db", NULL);
+    g_unlink (fn);
+    g_free (fn);
 
     /* Make a new EL instance for the tests (this will repopulate the DB) */
     el = rtcom_el_new ();
@@ -203,8 +180,10 @@ START_TEST(test_add_event)
     RTComElQuery * query = NULL;
     RTComElEvent * ev = NULL;
     gint event_id = -1;
+    gint service_id = -1;
+    gint event_type_id = -1;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    GHashTable * values = NULL;
     time_t t = 0;
 
     t = time (NULL);
@@ -256,10 +235,11 @@ START_TEST(test_add_event)
 
     fail_unless(it != NULL, "Failed to get iterator");
 
-    rtcom_fail_unless_strcmp(rtcom_el_iter_get_service(it), ==,
-            SERVICE);
-    rtcom_fail_unless_strcmp(rtcom_el_iter_get_event_type(it), ==,
-            EVENT_TYPE);
+    fail_unless(rtcom_el_iter_get_values(it, "service-id",
+        &service_id, "event-type-id", &event_type_id, NULL));
+
+    fail_unless (rtcom_el_get_service_id(el, SERVICE) == service_id);
+    fail_unless (rtcom_el_get_eventtype_id(el, EVENT_TYPE) == event_type_id);
 
     /* exercise its GObject properties */
     {
@@ -281,7 +261,6 @@ START_TEST(test_add_event)
                 "query", &it_query,
                 "sqlite3-database", &it_db,
                 "sqlite3-statement", &it_stmt,
-                "sql", &it_sql,
                 "plugins-table", &it_plugins,
                 "atomic", &it_atomic,
                 NULL);
@@ -291,7 +270,6 @@ START_TEST(test_add_event)
         fail_unless (it_db != NULL);
         fail_unless (it_db == el_db);
         fail_unless (it_stmt != NULL);
-        fail_unless (it_sql != NULL);
         fail_unless (it_plugins != NULL);
         fail_unless (it_atomic == TRUE || it_atomic == FALSE);
 
@@ -300,7 +278,7 @@ START_TEST(test_add_event)
 
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
+    values = rtcom_el_iter_get_value_map(
             it,
             "start-time",
             "end-time",
@@ -321,37 +299,33 @@ START_TEST(test_add_event)
     }
 
     rtcom_fail_unless_intcmp(t, ==,
-            g_value_get_int(g_value_array_get_nth(values, 0)));
+            g_value_get_int(g_hash_table_lookup(values, "start-time")));
     rtcom_fail_unless_intcmp(t, ==,
-            g_value_get_int(g_value_array_get_nth(values, 1)));
+            g_value_get_int(g_hash_table_lookup(values, "end-time")));
 
     rtcom_fail_unless_intcmp(FLAGS, ==,
-            g_value_get_int(g_value_array_get_nth(values, 2)));
+            g_value_get_int(g_hash_table_lookup(values, "flags")));
 
     rtcom_fail_unless_intcmp(BYTES_SENT, ==,
-            g_value_get_int(g_value_array_get_nth(values, 3)));
+            g_value_get_int(g_hash_table_lookup(values, "bytes-sent")));
     rtcom_fail_unless_intcmp(BYTES_RECEIVED, ==,
-            g_value_get_int(g_value_array_get_nth(values, 4)));
+            g_value_get_int(g_hash_table_lookup(values, "bytes-received")));
     rtcom_fail_unless_strcmp(LOCAL_UID, ==,
-            g_value_get_string(g_value_array_get_nth(values, 5)));
+            g_value_get_string(g_hash_table_lookup(values, "local-uid")));
     rtcom_fail_unless_strcmp(LOCAL_NAME, ==,
-            g_value_get_string(g_value_array_get_nth(values, 6)));
+            g_value_get_string(g_hash_table_lookup(values, "local-name")));
     rtcom_fail_unless_strcmp(REMOTE_UID, ==,
-            g_value_get_string(g_value_array_get_nth(values, 7)));
+            g_value_get_string(g_hash_table_lookup(values, "remote-uid")));
     rtcom_fail_unless_strcmp(REMOTE_NAME, ==,
-            g_value_get_string(g_value_array_get_nth(values, 8)));
+            g_value_get_string(g_hash_table_lookup(values, "remote-name")));
     rtcom_fail_unless_strcmp(CHANNEL, ==,
-            g_value_get_string(g_value_array_get_nth(values, 9)));
+            g_value_get_string(g_hash_table_lookup(values, "channel")));
     rtcom_fail_unless_strcmp(FREE_TEXT, ==,
-            g_value_get_string(g_value_array_get_nth(values, 10)));
+            g_value_get_string(g_hash_table_lookup(values, "free-text")));
 
     fail_if(rtcom_el_iter_next(it), "Iterator should only return one row");
 
-    fail_unless(rtcom_el_iter_refresh(it),
-            "Refreshing should work and return one row");
-    fail_if(rtcom_el_iter_next(it), "Iterator should only return one row");
-
-    g_value_array_free(values);
+    g_hash_table_destroy(values);
     g_object_unref(it);
     rtcom_el_event_free_contents (ev);
     rtcom_el_event_free (ev);
@@ -366,7 +340,6 @@ START_TEST(test_add_full)
     RTComElIter *it;
     RTComElAttachIter *att_it;
     RTComElAttachment *att;
-    GValueArray *values;
     GHashTable *headers;
     GList *attachments = NULL;
     gint fd;
@@ -427,10 +400,9 @@ START_TEST(test_add_full)
     fail_unless (it != NULL, "Failed to get iterator");
     fail_unless (rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(it, HEADER_KEY, NULL);
-    rtcom_fail_unless_strcmp ("add_full", ==,
-            g_value_get_string (g_value_array_get_nth (values, 0)));
-    g_value_array_free(values);
+    fail_unless (rtcom_el_iter_get_values (it, HEADER_KEY, &contents, NULL));
+    rtcom_fail_unless_strcmp ("add_full", ==, contents);
+    g_free (contents);
 
     att_it = rtcom_el_iter_get_attachments(it);
     fail_unless (att_it != NULL, "Failed to get attachment iterator");
@@ -490,8 +462,8 @@ START_TEST(test_header)
     gint event_id = -1;
     gint header_id = -1;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
     GHashTable *headers;
+    gchar *contents;
 
     ev = event_new_lite ();
     if(!ev)
@@ -524,17 +496,12 @@ START_TEST(test_header)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(it, HEADER_KEY, NULL);
-    fail_if (values == NULL, "Failed to get values");
+    fail_unless(rtcom_el_iter_get_values(it, HEADER_KEY, &contents, NULL));
+    rtcom_fail_unless_strcmp(HEADER_VAL, ==, contents);
+    g_free (contents);
 
-    rtcom_fail_unless_strcmp(HEADER_VAL, ==,
-            g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    fail_unless(rtcom_el_iter_refresh(it),
-            "Refreshing should work and return one row");
     fail_if(rtcom_el_iter_next(it), "Iterator should only return one row");
 
-    g_value_array_free(values);
     g_object_unref(it);
     rtcom_el_event_free_contents (ev);
     rtcom_el_event_free (ev);
@@ -683,8 +650,8 @@ START_TEST(test_read)
     RTComElEvent * ev = NULL;
     gint event_id = -1;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
     gint i;
+    gboolean is_read;
     gint count;
     gint ids[4] = { 0, 0, 0, 0 }; /* three IDs plus 0-terminator */
 
@@ -717,15 +684,8 @@ START_TEST(test_read)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-        it,
-            "is-read",
-            NULL);
-
-    fail_if (values == NULL, "Failed to get values");
-
-    fail_unless (g_value_get_boolean(g_value_array_get_nth(values, 0)) == TRUE,
-                 "is-read flag doesn't match");
+    fail_unless(rtcom_el_iter_get_values(it, "is-read", &is_read, NULL));
+    fail_unless(is_read == TRUE, "is-read flag doesn't match");
 
     /* At this point exactly one event has is-read = TRUE */
     count = iter_count_results (it);
@@ -753,7 +713,7 @@ START_TEST(test_read)
             fail_unless (rtcom_el_iter_next (it));
         }
 
-        fail_unless (rtcom_el_iter_get_int (it, "id", ids + i));
+        fail_unless (rtcom_el_iter_get_values (it, "id", ids + i, NULL));
     }
 
     fail_if (rtcom_el_iter_next (it), "Iterator should run out after 3");
@@ -787,7 +747,6 @@ START_TEST(test_read)
     g_object_unref (query);
     fail_unless (it == NULL, "all read flags should have been unset");
 
-    g_value_array_free(values);
     rtcom_el_event_free_contents (ev);
     rtcom_el_event_free (ev);
 }
@@ -799,7 +758,6 @@ START_TEST(test_flags)
     RTComElEvent * ev = NULL;
     gint event_id = -1;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
     gint test_flag1 = 0;
     gint flags = 0;
 
@@ -830,19 +788,12 @@ START_TEST(test_flags)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-        it,
-        "flags",
-        NULL);
-
-    fail_if (values == NULL, "Failed to get values");
+    fail_unless(rtcom_el_iter_get_values(it, "flags", &flags, NULL));
 
     test_flag1 = rtcom_el_get_flag_value(el, "RTCOM_EL_FLAG_TEST_FLAG1");
-    flags = g_value_get_int(g_value_array_get_nth(values, 0));
 
     fail_if ((flags & test_flag1) == 0, "flags don't match");
 
-    g_value_array_free(values);
     g_object_unref(it);
     rtcom_el_event_free_contents (ev);
     rtcom_el_event_free (ev);
@@ -885,7 +836,7 @@ START_TEST(test_get)
     result = rtcom_el_event_new ();
     fail_unless (result != NULL, "failed to create result event");
 
-    fail_unless (rtcom_el_iter_get (it, result),
+    fail_unless (rtcom_el_iter_get_full (it, result),
                  "Failed to get event from iterator");
 
     fail_unless (rtcom_el_event_equals (ev, result),
@@ -1029,18 +980,12 @@ START_TEST(test_get_string)
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
     bar = GUINT_TO_POINTER (0xDEADBEEF);
-    fail_if(rtcom_el_iter_dup_string(it, "bytes-sent", &bar),
-            "Shouldn't be able to get an int value as a string");
-    fail_unless(bar == GUINT_TO_POINTER (0xDEADBEEF),
-            "bar should be left untouched in this case");
-
-    bar = GUINT_TO_POINTER (0xDEADBEEF);
-    fail_if(rtcom_el_iter_dup_string(it, "there is no such key", &bar),
+    fail_if(rtcom_el_iter_get_values(it, "there is no such key", &bar, NULL),
             "Shouldn't be able to get a missing value as a string");
     fail_unless(bar == GUINT_TO_POINTER (0xDEADBEEF),
             "bar should be left untouched in this case");
 
-    fail_unless(rtcom_el_iter_dup_string(it, HEADER_KEY,  &bar));
+    fail_unless(rtcom_el_iter_get_values(it, HEADER_KEY,  &bar, NULL));
     fail_if(bar == NULL);
     fail_if(bar == GUINT_TO_POINTER (0xDEADBEEF));
     rtcom_fail_unless_strcmp(bar, ==, HEADER_VAL);
@@ -1084,20 +1029,10 @@ START_TEST(test_get_int)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    fail_unless(rtcom_el_iter_get_int(it, "bytes-sent", &retrieved),
+    fail_unless(rtcom_el_iter_get_values(it, "bytes-sent", &retrieved, NULL),
             "Failed to get bytes-sent");
     rtcom_fail_unless_intcmp(retrieved, ==, BYTES_SENT);
 
-    retrieved = 12345;
-    fail_if(rtcom_el_iter_get_int(it, "channel", &retrieved),
-            "Shouldn't be able to get a string value as an int");
-    rtcom_fail_unless_intcmp(retrieved, ==, 12345); /* should be untouched */
-
-    retrieved = 12345;
-    fail_if(rtcom_el_iter_get_int(it, "there is no such key", &retrieved),
-            "Shouldn't be able to get a missing value as an int");
-    rtcom_fail_unless_intcmp(retrieved, ==, 12345); /* should be untouched */
-
     g_object_unref(it);
     rtcom_el_event_free_contents (ev);
     rtcom_el_event_free (ev);
@@ -1108,7 +1043,7 @@ START_TEST(test_ends_with)
 {
     RTComElQuery * query = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    gchar *contents;
     time_t t = 0;
 
     t = time (NULL);
@@ -1128,43 +1063,24 @@ START_TEST(test_ends_with)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
-
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL));
 
     /* It is an API guarantee that we're sorting by event ID, which ascends
      * as time goes on, so Eve's recent message comes before Dave's older
      * message */
-    rtcom_fail_unless_strcmp("I am online", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    g_value_array_free(values);
+    rtcom_fail_unless_strcmp("I am online", ==, contents);
+    g_free(contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL));
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    rtcom_fail_unless_strcmp("Hello from Dave", ==, contents);
 
-    rtcom_fail_unless_strcmp("Hello from Dave", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
+    g_free(contents);
 
     fail_if (rtcom_el_iter_next (it));
 
-    g_value_array_free(values);
-
     g_object_unref(it);
 }
 END_TEST
@@ -1245,8 +1161,8 @@ START_TEST(test_in_strv)
 {
     RTComElQuery * query = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
     time_t t = 0;
+    gchar *contents;
     const gchar * const interesting_people[] = { "Chris", "Dave", NULL };
 
     t = time (NULL);
@@ -1266,43 +1182,25 @@ START_TEST(test_in_strv)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
-
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL));
 
     /* It is an API guarantee that we're sorting by event ID, which ascends
      * as time goes on, so Dave's recent message comes before Chris's older
      * message */
 
-    rtcom_fail_unless_strcmp("Hello from Dave", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
+    rtcom_fail_unless_strcmp("Hello from Dave", ==, contents);
 
-    g_value_array_free(values);
+    g_free (contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL));
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
-
-    rtcom_fail_unless_strcmp("Hello from Chris", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
+    rtcom_fail_unless_strcmp("Hello from Chris", ==, contents);
 
     fail_if (rtcom_el_iter_next (it));
 
-    g_value_array_free(values);
+    g_free (contents);
     g_object_unref(it);
 }
 END_TEST
@@ -1372,7 +1270,7 @@ START_TEST(test_string_equals)
 {
     RTComElQuery * query = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    gchar *contents;
     time_t t = 0;
 
     t = time (NULL);
@@ -1393,43 +1291,24 @@ START_TEST(test_string_equals)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
-
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+        "Failed to get values.");
 
     /* It is an API guarantee that we're sorting by event ID, which ascends
      * as time goes on, so Bob's recent message comes before his older
      * message */
 
-    rtcom_fail_unless_strcmp("Are you there?", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    g_value_array_free(values);
+    rtcom_fail_unless_strcmp("Are you there?", ==, contents);
+    g_free(contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+        "Failed to get values.");
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    rtcom_fail_unless_strcmp("Hi Alice", ==, contents);
+    g_free(contents);
 
-    rtcom_fail_unless_strcmp("Hi Alice", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    fail_if (rtcom_el_iter_next (it));
-
-    g_value_array_free(values);
     g_object_unref(it);
 }
 END_TEST
@@ -1438,7 +1317,7 @@ START_TEST(test_int_ranges)
 {
     RTComElQuery * query = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    gchar *contents;
     time_t t = 0;
 
     t = time (NULL);
@@ -1459,76 +1338,41 @@ START_TEST(test_int_ranges)
     fail_unless(it != NULL, "Failed to get iterator");
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
-
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+            "Failed to get values");
 
     /* It is an API guarantee that we're sorting by event ID, which ascends
      * as time goes on */
 
-    rtcom_fail_unless_strcmp("Are you there?", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    g_value_array_free(values);
+    rtcom_fail_unless_strcmp("Are you there?", ==, contents);
+    g_free(contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+            "Failed to get values");
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
-
-    rtcom_fail_unless_strcmp("Hello from Dave", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    g_value_array_free(values);
+    rtcom_fail_unless_strcmp("Hello from Dave", ==, contents);
+    g_free(contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+            "Failed to get values");
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
-
-    rtcom_fail_unless_strcmp("Hello from Chris", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
-
-    g_value_array_free(values);
+    rtcom_fail_unless_strcmp("Hello from Chris", ==, contents);
+    g_free(contents);
 
     fail_unless (rtcom_el_iter_next (it));
 
-    values = rtcom_el_iter_get_valuearray(
-            it,
-            "free-text",
-            NULL);
+    fail_unless(rtcom_el_iter_get_values(it, "free-text", &contents, NULL),
+            "Failed to get values");
 
-    if(!values)
-    {
-        fail("Failed to get values.");
-    }
-
-    rtcom_fail_unless_strcmp("Hi Alice", ==,
-               g_value_get_string(g_value_array_get_nth(values, 0)));
+    rtcom_fail_unless_strcmp("Hi Alice", ==, contents);
+    g_free(contents);
 
     fail_if (rtcom_el_iter_next (it));
 
-    g_value_array_free(values);
     g_object_unref(it);
 }
 END_TEST
@@ -1554,38 +1398,38 @@ START_TEST(test_group_by_uids)
      * as time goes on: so this is the order we'll get */
 
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("bob@example.com", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "local-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "local-uid", &s, NULL));
     rtcom_fail_unless_strcmp("butterfly/msn/alice", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("christine@msn.invalid", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("eve@example.com", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("bob@example.com", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "free-text", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "free-text", &s, NULL));
     rtcom_fail_unless_strcmp("Are you there?", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("dave@example.com", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("chris@example.com", ==, s);
     g_free (s);
 
@@ -1618,20 +1462,20 @@ START_TEST(test_group_by_metacontacts)
      * as time goes on: so this is the order we'll get */
 
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("bob@example.com", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "local-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "local-uid", &s, NULL));
     rtcom_fail_unless_strcmp("butterfly/msn/alice", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("christine@msn.invalid", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("eve@example.com", ==, s);
     g_free (s);
 
@@ -1643,15 +1487,15 @@ START_TEST(test_group_by_metacontacts)
      * like AIM, Skype, Myspace, IRC etc.)
      */
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("bob@example.com", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "local-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "local-uid", &s, NULL));
     rtcom_fail_unless_strcmp("gabble/jabber/alice", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("dave@example.com", ==, s);
     g_free (s);
 
@@ -1690,24 +1534,24 @@ START_TEST(test_group_by_group)
      * as time goes on: so this is the order we'll get */
 
     fail_unless(rtcom_el_iter_first(it), "Failed to start iterator");
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("bob@example.com", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "local-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "local-uid", &s, NULL));
     rtcom_fail_unless_strcmp("butterfly/msn/alice", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "group-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "group-uid", &s, NULL));
     rtcom_fail_unless_strcmp("group(bob)", ==, s);
     g_free (s);
 
     fail_unless (rtcom_el_iter_next (it));
-    fail_unless (rtcom_el_iter_dup_string (it, "remote-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "remote-uid", &s, NULL));
     rtcom_fail_unless_strcmp("frank@msn.invalid", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "local-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "local-uid", &s, NULL));
     rtcom_fail_unless_strcmp("butterfly/msn/alice", ==, s);
     g_free (s);
-    fail_unless (rtcom_el_iter_dup_string (it, "group-uid", &s));
+    fail_unless (rtcom_el_iter_get_values (it, "group-uid", &s, NULL));
     rtcom_fail_unless_strcmp("group(chris+frank)", ==, s);
     g_free (s);
 
@@ -1749,8 +1593,11 @@ START_TEST(test_update_remote_contact)
                 "butterfly/msn/alice", "bob@example.com",
                 "abook-bob", "Robert", NULL));
 
+    g_object_unref (it);
+    it = rtcom_el_get_events (el, query_by_abook);
+    fail_unless (it != NULL, "Failed to get iterator");
+
     /* Bob's MSN message is attached to that uid too */
-    fail_unless (rtcom_el_iter_refresh (it));
     count = iter_count_results (it);
     rtcom_fail_unless_intcmp (count, ==, 3);
 
@@ -1774,7 +1621,11 @@ START_TEST(test_update_remote_contact)
     fail_unless (rtcom_eventlogger_update_remote_contact (el,
                 "butterfly/msn/alice", "bob@example.com",
                 NULL, "Robert", NULL));
-    fail_unless (rtcom_el_iter_refresh (it));
+
+    g_object_unref (it);
+    it = rtcom_el_get_events (el, query_by_name);
+    fail_unless (it != NULL, "Failed to get iterator");
+
     count = iter_count_results (it);
     rtcom_fail_unless_intcmp (count, ==, 3);
 
@@ -1785,11 +1636,16 @@ START_TEST(test_update_remote_contact)
 }
 END_TEST
 
+extern void db_extend_el_suite (Suite *s);
+
 Suite *
 el_suite(void)
 {
     Suite * s = suite_create ("rtcom-eventlogger");
 
+    /* Low-level DB APi test cases */
+    db_extend_el_suite(s);
+
     /* Core test case */
     TCase * tc_core = tcase_create("Core");
     tcase_add_checked_fixture(tc_core, core_setup, core_teardown);
